#include "Utils.h"

#define PROGRESS_LENGTH 5

/**
* Interrupt routine that can be used in the samples
* If you register this interrupt routine thanks to the function i_PCIe3521_SetBoardIntRoutine, this function
* will be called each time the board generates an interruption
**/
void InterruptRoutine (HANDLE	deviceHandle,
					   BYTE		fifoOverflow,
					   DWORD	interruptSource,
					   DWORD	interruptMask,
					   PWORD	args,
					   DWORD	argsCount)
{
	UNREFERENCED_PARAMETER(deviceHandle);
	/* switch through the possible interrupt sources */

	/* check the interrupt fifo overflow */
	if (fifoOverflow)
	{
		printf("Error: interrupt fifo overflow. That means that your software do not handle the interrupts fast enough.\n");
	}

	/* end of sequence */
	if (interruptSource & 0x1)
	{
		DWORD i = 0;
		printf("End of sequence interrupt\n");
		/* display the values */
		printf("Value(s) : \t");
		for (i = 0 ; i < argsCount ; i++)
		{
			printf("0x%.4x \t", args[i]);
		}
		printf("\n");
	}

	/* timer */
	if (interruptSource & 0x2)
	{
		printf("Interrupt from timer %d!                                            \n", interruptMask);
	}

	/* DMA */
	if (interruptSource & 0x4)
	{
		if (interruptMask & 0x4)
		{
			printf("internal sequence fifo overflow \n");
		}
		else
		{
			printf("dma interrupt \n");
		}
	}

	/* DMA counter overflow */
	if (interruptSource & 0x8)
	{
		printf("DMA counter overflow. This stop the acquisition.\n");
	}

	/* PLD Fifo overflow */
	if (interruptSource & 0x10)
	{
		printf("Error: PLD Fifo overflow. This stop the acquisition.\n");
	}
}

/**
* Enable to read a ulong from the keyboard
* @param description: A description string that is printed to give some information about the value that must be given
**/
void ReadUlong(char* description, ULONG minValue, ULONG maxValue, ULONG* readValue)
{
	do
	{
		printf("%s", description);
		_flushall();
		scanf_s("%u", readValue);
	}
	while((*readValue < minValue) || (*readValue > maxValue));
}

/**
* Enable to display informations about a board, given its index
**/
static INT DisplayBoardInformations (BYTE boardIndex)
{
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
	DWORD identifierSize = 50;
	CHAR identifier[50] = {0};
	DWORD uiNumber = 0;
	DWORD deviceNumber = 0;
	DWORD busNumber = 0;
	DWORD baseAddress0 = 0;
	DWORD baseAddress1 = 0;
	DWORD baseAddress2 = 0;
	DWORD baseAddress3 = 0;
	BYTE interrupt = 0;
	DWORD deviceDriverVersion = 0;

	/* Get informations about the board */
	returnValue = i_PCIe3521_GetBoardInformation(boardIndex,
							identifierSize,
							identifier,
							&uiNumber,
							&deviceNumber,
							&busNumber,
							&baseAddress0,
							&baseAddress1,
							&baseAddress2,
							&baseAddress3,
							&interrupt);

	/* Error handling */
	if (returnValue != 0)
	{
		printf("error getting board information : %d \n", returnValue);
		return -1;
	}

	/* 
	* Open an handle to the board 
	* That will enable to get the version of the device driver applied to this device
	*/
	returnValue = i_PCIe3521_OpenBoardViaIndex(boardIndex, &boardHandle);

	/* Error handling */
	if (returnValue != 0)
	{
		printf("error opening board : %d \n", returnValue);
		return -2;
	}

	/* Get the driver version */
	returnValue = i_PCIe3521_GetDeviceDriverVersion(boardHandle, &deviceDriverVersion);
	if (returnValue != 0)
	{
		printf("error getting device driver version : %d \n", returnValue);
		return -3;
	}

	/* Display informations about the board */
	printf("**********************************************\n");
	printf("Board index : %d \n", boardIndex);
	printf("Identifier : %s \n", identifier);
	printf("Base address 0 : %x \n", baseAddress0);
	printf("Base address 1 : %x \n", baseAddress1);
	printf("Base address 2 : %x \n", baseAddress2);
	printf("Base address 3 : %x \n", baseAddress3);
	printf("UI Number : %d \n", (int)uiNumber);
	printf("Device Number : %d \n", (int)deviceNumber);
	printf("Bus Number : %d \n", (int)busNumber);
	printf("Interrupt line : %d \n", interrupt);
	printf("Device driver version : 0x%0.8x \n", (int)deviceDriverVersion);

	/* Close the board */
	returnValue = i_PCIe3521_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("error closing the board: %d \n", returnValue);
		return -4;
	}

	return 0;
}

/**
* Method used to check that the index given by the user is correct
**/
static inline BOOL IsIndexCorrect (BYTE boardNbr, int index)
{
	if (index < 0)
	{
		return FALSE;
	}
	if (index >= boardNbr)
	{
		return FALSE;
	}
	return TRUE;
}

/**
* Enable to open a board via index
* The index is asked to the user
**/
static INT OpenBoardViaIndex(HANDLE * deviceHandle , BYTE boardNbr)
{
	INT returnValue = 0;
	int boardIndex = 0xFF;

	/* Get the index from the user */
	do
	{
		printf("Select the board you want to use (0 to %d) ", (int)(boardNbr-1));
		_flushall();
		scanf_s("%d",&boardIndex);
	} while (IsIndexCorrect(boardNbr, boardIndex) == FALSE);

	/* Open the board */
	returnValue = i_PCIe3521_OpenBoardViaIndex((BYTE)boardIndex, deviceHandle);

	/* error handling */
	if (returnValue != 0)
	{
		printf("Error while opening the board via index : %d \n", returnValue);
		return -1;
	}

	/* success */
	return 0;
}

/**
* Enable to open a board via identifier
* The identifier is asked to the user
**/
static INT OpenBoardViaIdentifier (HANDLE * deviceHandle)
{
	INT returnValue = 0;
	CHAR identifierChoice[50] = {0};

	/* Ask for the identifier choice */
	printf("Enter the selected identifier : ");
	_flushall();
	scanf_s("%s",identifierChoice);

	returnValue = i_PCIe3521_OpenBoardViaIdentifier(identifierChoice, deviceHandle);
	
	/* Error handling */
	if (returnValue != 0)
	{
		printf("error opening board via identifier: %d \n", returnValue);
		return -1;
	}

	/* success */
	return 0;
}

/**
* Enables to give to the user the choice to open a board
* The function also enables (first) to display informations about the available boards
**/
INT OpenBoard(HANDLE * deviceHandle)
{
	BYTE boardNbr = 0;
	INT returnValue = 0;
	DWORD dllVersion = 0;
	BYTE i = 0;
	int openMode = -1;

	/* Get the version of the DLL */
	returnValue = i_PCIe3521_GetDLLVersion(&dllVersion);
	
	/* Error handling */
	if (returnValue != 0)
	{
		printf("error getting the DLL's version : %d \n", returnValue);
		return -1;
	}
	/* Print the DLL version */
	printf("DLL version : 0x%0.8x \n", (int)dllVersion);

	/* Get the number of boards available */
	returnValue = i_PCIe3521_GetNumberOfBoards(&boardNbr);

	/* Error handling */
	if (returnValue != 0)
	{
		printf("error getting number of boards : %d \n", returnValue);
		return -2;
	}

	/* Check the number of board */
	if (boardNbr == 0)
	{
		printf("Error : no board found \n");
		return -3;
	}

	/* Loop through all the boards to display useful informations */
	for (i = 0; i < boardNbr; i++)
	{
		/* Display informations about the board */
		returnValue = DisplayBoardInformations(i);

		if (returnValue != 0)
		{
			printf("Error displaying board informations : %d \n", (int)returnValue);
			return -4;
		}
	}

	/* Ask opening mode : index or identifier */
	do
	{
		printf("Do you want to open the driver via index (0) or via identifier (1): ");
		_flushall();
		scanf_s("%d",&openMode);
	} while ((openMode != 0) && (openMode != 1));

	/* call the correct function */
	switch (openMode)
	{
		case 0:
			return OpenBoardViaIndex(deviceHandle, boardNbr);
		case 1:
			return OpenBoardViaIdentifier(deviceHandle);
		default:
			/* unknown mode */
			return -1;
	}
}

/**
* Display a sort of progress bar
**/
void DisplayRunningStatus()
{
	static int charNbr = 0;
	int i = 0;

	/* display running status */
	for (i = 0; i < PROGRESS_LENGTH; i++)
	{
		if (i < charNbr) 
		{
			printf(".");
		}
		else
		{
			printf(" ");
		}
	}
	printf("\r");
	charNbr = (charNbr > PROGRESS_LENGTH) ? 0 : charNbr + 1;
}