using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCIe3521_NET
{
    public class PCIe3521
    {
        /// <summary>
        /// Store the index of the board
        /// </summary>
        private int index;

        /// <summary>
        /// Store an handle to the board. Will be used for interrupt routine for example
        /// </summary>
        private IntPtr handle;

        /// <summary>
        /// An object that will contain the timer event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private TimerEventArgs timerEventArgs;

        /// <summary>
        /// Handler to the interrupt function of the board
        /// The function pointed by this delegate will be called each time the board interrupts
        /// </summary>
        private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

        #region "Event"

        /// <summary>
        /// The timer event
        /// </summary>
        public event TimerEventHandler TimerEventReceived;

        /// <summary>
        /// Delegate to provide when an interrupt on the timer occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void TimerEventHandler(object source, TimerEventArgs args);

        #endregion

        #region "Constructor/Destructor"

        /// <summary>
        /// Constructor of the board
        /// Cannot be called; initialize the internal variables
        /// </summary>
        private PCIe3521()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args (once by board)
            this.timerEventArgs = new TimerEventArgs();

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);
        }

        /// <summary>
        /// Destructor of the board
        /// Free all the ressources
        /// </summary>
        ~PCIe3521()
        {
            DllImport.i_PCIe3521_CloseBoard(handle);
            this.index = -1;
            this.handle = IntPtr.Zero;
        }

        #endregion

        #region "Utils"

        /// <summary>
        /// Enables to get an instance of a PCIe3521 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCIe3521 on success</returns>
        public static PCIe3521 OpenBoard(int index)
        {
            PCIe3521 boardInstance = null;
            int nbBoard = 0;
            IntPtr handle = IntPtr.Zero;

            // Get the number of boards
            if ((nbBoard = GetBoardNumber()) == -1)
            {
                Console.WriteLine("GetBoardNumber failed");
                return null;
            }

            // Check that the index is in the range
            if (index < 0 || index > nbBoard)
            {
                Console.WriteLine("index not valid");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCIe3521_OpenBoardViaIndex((byte)index, ref handle) != 0)
            {
                Console.WriteLine("i_PCIe3521_OpenBoardViaIndex failed");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCIe3521();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Enables to get the number of PCIe3521 boards plugged in the computer
        /// </summary>
        /// <returns>The number of board</returns>
        public static int GetBoardNumber()
        {
            int ret = 0;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCIe3521_GetNumberOfBoards(ref nbBoard);

            // Error case
            if (ret != 0) { return -1; }

             // Success
            else { return (int)nbBoard; }
        }

        #endregion

        #region "Interrupt"

        /// <summary>
        /// Interrupt function - Called each time the board generates an interrupt
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="overflow"></param>
        /// <param name="source"></param>
        /// <param name="mask"></param>
        /// <param name="args"></param>
        /// <param name="argsSize"></param>
        private void InterruptFunction(IntPtr handle, byte overflow,uint source,uint mask,IntPtr args, uint argsSize )
        {
            // Timer ?
            if ((source & 2) == 2)
            {
                this.timerEventArgs.InterruptMask = mask;
                this.timerEventArgs.FifoOverflow = overflow;

                // Signal the event
                this.OnTimerEventReceived(this.timerEventArgs);
            }
        }

        /// <summary>
        /// Set the board interrupt routine
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe3521_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        /// <returns></returns>
        public int ResetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe3521_ResetBoardIntRoutine(this.handle);
        }

        #endregion

        #region "Digital outputs"

        /// <summary>
        /// Set the digital memory of the outputs to on
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetDigitalOutputMemoryOn()
        {
            return (int)DllImport.i_PCIe3521_SetDigitalOutputMemoryOn(this.handle);
        }

        /// <summary>
        /// Set the digital memory of the outputs to off
        /// </summary>
        /// <returns></returns>
        public int SetDigitalOutputMemoryOff()
        {
            return (int)DllImport.i_PCIe3521_SetDigitalOutputMemoryOff(this.handle);
        }

        /// <summary>
        /// Enables to set one channel to on (high state)
        /// </summary>
        /// <param name="channel">The index of the channel (start at 0)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set1DigitalOutputOn(byte channel)
        {
            return (int)DllImport.i_PCIe3521_Set1DigitalOutputOn(this.handle, channel);
        }

        /// <summary>
        /// Enables to set one channel to off (low state)
        /// </summary>
        /// <param name="channel">The index of the channel (start at 0)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set1DigitalOutputOff(byte channel)
        {
            return (int)DllImport.i_PCIe3521_Set1DigitalOutputOff(this.handle, channel);
        }

        /// <summary>
        /// Enables to set several digital outputs to on (high state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 15)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set4DigitalOutputsOn(byte value)
        {
            return (int)DllImport.i_PCIe3521_Set4DigitalOutputsOn(this.handle, value);
        }

        /// <summary>
        /// Enables to set several digital outputs to off (low state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 15)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set4DigitalOutputsOff(byte value)
        {
            return (int)DllImport.i_PCIe3521_Set4DigitalOutputsOff(this.handle, value);
        }

        /// Read 1 digital output status
        /// </summary>
        /// <param name="channel">Channel to be read (0 to 31)</param>
        /// <param name="outputStatus">Status of the digital outputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Get1DigitalOutputsStatus(byte channel, ref byte outputStatus)
        {
            return (int) DllImport.i_PCIe3521_Get1DigitalOutputStatus(this.handle, channel, ref outputStatus);
        }

        /// <summary>
        /// Read 16 digital outputs status
        /// </summary>
        /// <param name="value">Status of the digital outputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Get4DigitalOutputsStatus(ref byte outputStatus)
        {
            return (int)DllImport.i_PCIe3521_Get4DigitalOutputsStatus(this.handle, ref outputStatus);
        }

        #endregion

        #region "Digital inputs"

        /// <summary>
        /// Enables to read one digital
        /// </summary>
        /// <param name="channel">Channel to read (0-8)</param>
        /// <param name="inputValue">A byte that will be filled with the current status of the input (0,1)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read1DigitalInput(byte channel, ref byte inputValue)
        {
            // Call the function
            return DllImport.i_PCIe3521_Read1DigitalInput(this.handle, channel, ref inputValue);
        }

        /// <summary>
        /// Enables to read 4 digital inputs (per port)
        /// </summary>
        /// <param name="inputValues">A byte that will be filled with the current status of the inputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read4DigitalInputs(ref byte inputValues)
        {
            // Call the function
            return DllImport.i_PCIe3521_Read4DigitalInputs(this.handle, ref inputValues);
        }

        #endregion

        #region "Timer"

        /// <summary>
        /// Enables to signal to all the listeners that an event on the timer occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnTimerEventReceived(TimerEventArgs eventArgs)
        {
            if (this.TimerEventReceived != null)
            {
                this.TimerEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Initialize and start the timer
        /// </summary>
        /// <param name="timerNb">which timer should be started (0-1)</param>
        /// <param name="mode">0: TIMER_MODE_2, 1: TIMER_MODE_3</param>
        /// <param name="timeBase">0: us, 1: ms, 2: sec</param>
        /// <param name="outputMode">0: disable, 1: enabled, 2: inverted</param>
        /// <param name="interruptFlag">0: no interrupt, 1: use interrupt</param>
        /// <param name="reloadValue">maximum value of the timer</param>
        /// <param name="watchdogEnabled">0: watchdog is disabled, 1: watchdog is enabled</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitAndStartTimer(byte timerNb, byte mode, byte timeBase,byte outputMode, byte interruptFlag, ushort reloadValue, byte watchdogEnabled)
        {
            return DllImport.i_PCIe3521_InitAndStartTimer(this.handle, timerNb, mode, timeBase, outputMode, interruptFlag, reloadValue, watchdogEnabled);
        }

        /// <summary>
        /// Stop the timer
        /// </summary>
        /// <param name="timerNb">which timer should be stopped (0-1)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int StopTimer(byte timerNb)
        {
            return DllImport.i_PCIe3521_StopTimer(this.handle, timerNb);
        }

        /// <summary>
        /// Read the current status of the timer
        /// </summary>
        /// <param name="timerNb">which timer should be read (0-1)</param>
        /// <param name="status">0: stopped, 1 started</param>
        /// <param name="softwareTriggerStatus">0: no software trigger, 1 software trigger occured</param>
        /// <param name="value">current value of the timer</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadTimerStatus(byte timerNb, ref byte status, ref byte softwareTriggerStatus, ref ushort value)
        {
            return DllImport.i_PCIe3521_ReadTimerStatus(this.handle, timerNb, ref status, ref softwareTriggerStatus, ref value);
        }

        /// <summary>
        /// Trigger the timer
        /// </summary>
        /// <param name="timerNb">which timer should be triggered (0-1)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int TriggerTimer(byte timerNb)
        {
            return DllImport.i_PCIe3521_TriggerTimer(this.handle, timerNb);
        }

        #endregion

		#region Analog outputs
		
		/// <summary>
        /// Set one analog output
        /// </summary>
        /// <param name="channel">The analog channel to set</param>
		/// <param name="polarity">The polarity of the channel to set</param>
		/// <param name="value">The value to set</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Write1AnalogOutput(byte channel, byte polarity, ushort value)
        {
			return DllImport.i_PCIe3521_Write1AnalogOutput(this.handle, channel, polarity, value);		
        }

		/// <summary>
        /// Set several analog outputs
        /// </summary>
		/// <param name="channelCount">The number of analog channels to set</param>
        /// <param name="channelArray">The analog channels to set</param>
		/// <param name="polarityArray">The polarity of the channels to set</param>
		/// <param name="valueArray">The values to set</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int WriteMoreAnalogOutputs(byte channelCount, byte[] channelArray, byte[] polarityArray, ushort[] valueArray)
        {
			return DllImport.i_PCIe3521_WriteMoreAnalogOutputs(this.handle, channelCount, channelArray, polarityArray, valueArray);
        }
		
		#endregion
    }
}
